@@ -12,7 +12,10 @@
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
 # Copyright (c) 2020 Ram Rachum <ram@rachum.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2022 areveny <areveny@protonmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
@@ -21,25 +24,17 @@
 
 from functools import partial
 
-from astroid import (
-    arguments,
-    helpers,
-    inference_tip,
-    nodes,
-    objects,
-    scoped_nodes,
-    util,
-)
+from astroid import arguments, helpers, inference_tip, nodes, objects, util
 from astroid.builder import AstroidBuilder
 from astroid.exceptions import (
     AstroidTypeError,
     AttributeInferenceError,
     InferenceError,
     MroError,
-    NameInferenceError,
     UseInferenceDefault,
 )
 from astroid.manager import AstroidManager
+from astroid.nodes import scoped_nodes
 
 OBJECT_DUNDER_NEW = "object.__new__"
 
@@ -162,7 +157,7 @@ def _builtin_filter_predicate(node, builtin_name):
         and isinstance(node.parent, nodes.Assign)
         and len(node.parent.targets) == 1
         and isinstance(node.parent.targets[0], nodes.AssignName)
-        and node.parent.targets[0].name in ("Pattern", "Match")
+        and node.parent.targets[0].name in {"Pattern", "Match"}
     ):
         # Handle re.Pattern and re.Match in brain_re
         # Match these patterns from stdlib/re.py
@@ -331,7 +326,7 @@ def _get_elts(arg, context):
 
     try:
         inferred = next(arg.infer(context))
-    except (InferenceError, NameInferenceError, StopIteration) as exc:
+    except (InferenceError, StopIteration) as exc:
         raise UseInferenceDefault from exc
     if isinstance(inferred, nodes.Dict):
         items = inferred.items
@@ -572,7 +567,7 @@ def infer_property(node, context=None):
     if not isinstance(inferred, (nodes.FunctionDef, nodes.Lambda)):
         raise UseInferenceDefault
 
-    return objects.Property(
+    prop_func = objects.Property(
         function=inferred,
         name=inferred.name,
         doc=getattr(inferred, "doc", None),
@@ -580,6 +575,8 @@ def infer_property(node, context=None):
         parent=node,
         col_offset=node.col_offset,
     )
+    prop_func.postinit(body=[], args=inferred.args)
+    return prop_func
 
 
 def infer_bool(node, context=None):
@@ -675,9 +672,7 @@ def infer_issubclass(callnode, context=None):
         raise UseInferenceDefault("TypeError: issubclass() takes no keyword arguments")
     if len(call.positional_arguments) != 2:
         raise UseInferenceDefault(
-            "Expected two arguments, got {count}".format(
-                count=len(call.positional_arguments)
-            )
+            f"Expected two arguments, got {len(call.positional_arguments)}"
         )
     # The left hand argument is the obj to be checked
     obj_node, class_or_tuple_node = call.positional_arguments
@@ -722,9 +717,7 @@ def infer_isinstance(callnode, context=None):
         raise UseInferenceDefault("TypeError: isinstance() takes no keyword arguments")
     if len(call.positional_arguments) != 2:
         raise UseInferenceDefault(
-            "Expected two arguments, got {count}".format(
-                count=len(call.positional_arguments)
-            )
+            f"Expected two arguments, got {len(call.positional_arguments)}"
         )
     # The left hand argument is the obj to be checked
     obj_node, class_or_tuple_node = call.positional_arguments
