@@ -1,8 +1,10 @@
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+from typing import Optional
+
 from astroid import context, inference_tip, nodes
 from astroid.brain.helpers import register_module_extender
-from astroid.builder import extract_node, parse
+from astroid.builder import _extract_single_node, parse
 from astroid.const import PY37_PLUS, PY39_PLUS
 from astroid.manager import AstroidManager
 
@@ -60,11 +62,13 @@ def _looks_like_pattern_or_match(node: nodes.Call) -> bool:
         and isinstance(node.parent, nodes.Assign)
         and len(node.parent.targets) == 1
         and isinstance(node.parent.targets[0], nodes.AssignName)
-        and node.parent.targets[0].name in ("Pattern", "Match")
+        and node.parent.targets[0].name in {"Pattern", "Match"}
     )
 
 
-def infer_pattern_match(node: nodes.Call, ctx: context.InferenceContext = None):
+def infer_pattern_match(
+    node: nodes.Call, ctx: Optional[context.InferenceContext] = None
+):
     """Infer re.Pattern and re.Match as classes. For PY39+ add `__class_getitem__`."""
     class_def = nodes.ClassDef(
         name=node.parent.targets[0].name,
@@ -73,7 +77,7 @@ def infer_pattern_match(node: nodes.Call, ctx: context.InferenceContext = None):
         parent=node.parent,
     )
     if PY39_PLUS:
-        func_to_add = extract_node(CLASS_GETITEM_TEMPLATE)
+        func_to_add = _extract_single_node(CLASS_GETITEM_TEMPLATE)
         class_def.locals["__class_getitem__"] = [func_to_add]
     return iter([class_def])
 
