@@ -1,12 +1,16 @@
 import enum
 import sys
 
+PY36 = sys.version_info[:2] == (3, 6)
+PY38 = sys.version_info[:2] == (3, 8)
 PY37_PLUS = sys.version_info >= (3, 7)
 PY38_PLUS = sys.version_info >= (3, 8)
 PY39_PLUS = sys.version_info >= (3, 9)
 PY310_PLUS = sys.version_info >= (3, 10)
 BUILTINS = "builtins"  # TODO Remove in 2.8
 
+WIN32 = sys.platform == "win32"
+
 
 class Context(enum.Enum):
     Load = 1
