@@ -7,6 +7,7 @@
 # Copyright (c) 2017 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2017 rr- <rr-@sakuya.pl>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2021-2022 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
@@ -89,6 +90,7 @@ from astroid.nodes.node_classes import (  # pylint: disable=redefined-builtin (E
     Set,
     Slice,
     Starred,
+    Statement,
     Subscript,
     TryExcept,
     TryFinally,
@@ -112,11 +114,14 @@ from astroid.nodes.scoped_nodes import (
     GeneratorExp,
     Lambda,
     ListComp,
+    LocalsDictNodeNG,
     Module,
     SetComp,
     builtin_lookup,
     function_to_method,
+    get_wrapping_class,
 )
+from astroid.nodes.utils import Position
 
 _BaseContainer = BaseContainer  # TODO Remove for astroid 3.0
 
@@ -174,6 +179,7 @@ ALL_NODE_CLASSES = (
     Lambda,
     List,
     ListComp,
+    LocalsDictNodeNG,
     Match,
     MatchAs,
     MatchCase,
@@ -254,6 +260,7 @@ __all__ = (
     "FunctionDef",
     "function_to_method",
     "GeneratorExp",
+    "get_wrapping_class",
     "Global",
     "If",
     "IfExp",
@@ -265,6 +272,7 @@ __all__ = (
     "Lambda",
     "List",
     "ListComp",
+    "LocalsDictNodeNG",
     "Match",
     "MatchAs",
     "MatchCase",
@@ -281,12 +289,14 @@ __all__ = (
     "NodeNG",
     "Nonlocal",
     "Pass",
+    "Position",
     "Raise",
     "Return",
     "Set",
     "SetComp",
     "Slice",
     "Starred",
+    "Statement",
     "Subscript",
     "TryExcept",
     "TryFinally",
