@@ -4,14 +4,15 @@
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 Bryce Guinta <bryce.guinta@protonmail.com>
+# Copyright (c) 2021-2022 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
-
-import warnings
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 import importlib
+import warnings
+
 import lazy_object_proxy
 
 
@@ -56,8 +57,7 @@ class Uninferable:
     __nonzero__ = __bool__
 
     def accept(self, visitor):
-        func = getattr(visitor, "visit_uninferable")
-        return func(self)
+        return visitor.visit_uninferable(self)
 
 
 class BadOperationMessage:
@@ -82,7 +82,6 @@ class BadUnaryOperationMessage(BadOperationMessage):
         return helpers.object_type
 
     def _object_type(self, obj):
-        # pylint: disable=not-callable; can't infer lazy_import
         objtype = self._object_type_helper(obj)
         if objtype is Uninferable:
             return None
@@ -141,3 +140,16 @@ def proxy_alias(alias_name, node_type):
         },
     )
     return proxy(lambda: node_type)
+
+
+def check_warnings_filter() -> bool:
+    """Return True if any other than the default DeprecationWarning filter is enabled.
+
+    https://docs.python.org/3/library/warnings.html#default-warning-filter
+    """
+    return any(
+        issubclass(DeprecationWarning, filter[2])
+        and filter[0] != "ignore"
+        and filter[3] != "__main__"
+        for filter in warnings.filters
+    )
